/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.util;

//---IMPORTS---
import java.util.Date;
import java.text.DateFormat;
import java.util.Locale;

/**
 * This class provides static methods to format the
 * name of a calling object.  It's main use is by
 * the various objects within the system to create
 * a toString String to send to the logger.
 *
 * @author  $Author: tdb $
 * @version $Id: FormatName.java,v 1.15 2003/02/05 14:27:59 tdb Exp $
 */
public class FormatName {

//---FINAL ATTRIBUTES---
   
    /**
     * An array of names of verbosity levels.
     * Thus logging messages are now "classed" by the level
     */
    private final static String[] VERBOSITY_NAMES = {"FATAL  ", "ERROR  ", "WARNING", "SYSMSG ", "SYSINIT", "DEBUG  "};
   
//---STATIC METHODS---

    /**
     * This method takes a set of information about the calling 
     * class and constructs a tidy String name to be returned.
     * This is of use to the override of the toString() as
     * implemented by most of the iscream objects.
     *
     * @param friendlyName the configured name of the instance of the calling component the class is in (eg "filter1")
     * @param className the class name of the calling class, as obtained by getClass().getName()
     * @param revision the CVS Revision number for the calling class
     *
     * @return an iscream standard name to be used as a toString()
     */
    public static String getName(String friendlyName, String className, String revision) {
        if (friendlyName == null && className == null) {
            return "{static(v" + revision.substring(11, revision.length() - 2) + ")}";
        } else if (friendlyName == null) {
            return  "{"+ tidyClassName(className) + "(v" + revision.substring(11, revision.length() - 2) + ")}";
        } else if (className == null) {
            return friendlyName + "{static(v" + revision.substring(11, revision.length() - 2) + ")}";
        } else {
            return friendlyName + "{"+ tidyClassName(className) + "(v" + revision.substring(11, revision.length() - 2) + ")}";
        }
    }
 
    /**
     * This method generates a nicely formatted line for the log,
     * including the date/time and the source of the message. The date
     * and time are formatted using the DateFormat class, and the source
     * class is formatted using the toString() method found in every
     * source file. This is then prepended to the message and returned.
     * 
     * @param source A string representation of the calling object.
     * @param verbosity The verbosity of the message.
     * @param message The message to be logged.
     * @return The string to be written to the log.
     */
    public static String formatLogLine(String source, int verbosity, String message){
        String date = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.MEDIUM).format(new Date());
        return "[" + date + "] [" + VERBOSITY_NAMES[verbosity] + "] " + source + ": " + message;
    }

    /**
     * If the class name begins with uk.org.iscream.cms.server
     * this method will trim it off, otherwise it 
     * leaves the string unchanged.
     *
     * @param className the name of a class
     * 
     * @return the tidy version of it
     */
    private static String tidyClassName(String className) {
        String prefix = "uk.org.iscream.cms.server";
        if (className.startsWith(prefix)) {
            return className.substring(prefix.length()+1);
        }
        return className;
    }

//---CONSTRUCTORS---

    /**
     * A private constructor ensures an instance of this
     * class CANNOT be created.
     */
    private FormatName() {
        // do nothing on purpose!
    }

//---PUBLIC METHODS---

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

//---STATIC ATTRIBUTES---

}
