/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.util;

//---IMPORTS---
import java.net.*;
import java.io.*;

/**
 * The Simple Mail Transfer Protocol class. This class was inspired
 * by the GJT.
 *
 * @author  $Author: tdb $
 * @version $Id: Smtp.java,v 1.11 2003/02/05 14:27:59 tdb Exp $
 */
public class Smtp {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public static final String REVISION = "$Revision: 1.11 $";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

    /**
     * Connects to the specified SMTP server, on the default
     * (and standard) port 25.
     *
     * @param server The SMTP server to use
     * @throws IOException if the connection to the server fails
     */
    public Smtp(String server) throws IOException {
        this(server, 25);
    }
    
    /**
     * Connects to the specified SMTP server on a given port
     *
     * @param server The SMTP server to use
     * @param port The SMTP server port
     * @throws IOException if the connection to the server fails
     */
    public Smtp(String server, int port) throws IOException {
        _socket = new Socket(server, port);
        _socketIn = new BufferedReader(new InputStreamReader(_socket.getInputStream()));
        _socketOut = new PrintWriter(_socket.getOutputStream(), true);
        init();
    }

//---PUBLIC METHODS---

    /**
     * Closes down the connection to the server
     *
     * @throws IOException if the command fails
     */
    public void close() throws IOException {
        sendCommand("QUIT", 221);
        _socketIn.close();
        _socketOut.close();
        _socket.close();
    }
    
    /**
     * Specify who the message is from
     *
     * @param sender the e-mail address of the sender
     * @throws IOException if the command fails
     */
    public void setSender(String sender) throws IOException {
        sendCommand("MAIL FROM: <" + sender + ">", 250);
    }
    
    /**
      * Specify who the message is to be sent to
      *
      * @param to the e-mail address of the receiver
      * @throws IOException if the command fails
      */
    public void setTo(String to) throws IOException {
        sendCommand("RCPT TO: <" + to + ">", 250);
    }
    
    /**
     * Gets the PrintWriter allowing data to be sent
     * directly to the SMTP server
     *
     * @return a reference to the PrintWriter
     * @throws IOException if the command fails
     */
    public PrintWriter getOutputStream() throws IOException {
        sendCommand("DATA", 354);
        return _socketOut;
    }
    
    /**
     * Completes and sends the current message
     *
     * @throws IOException if the command fails
     */
    public void sendMessage() throws IOException {
        sendCommand(".", 250);
    }
    
    /**
     * Sends a command to the server
     *
     * @param cmd The command to send
     * @param reply The expected reply-code
     * @throws IOException if the incorrect response code is received
     */
    public void sendCommand(String cmd, int reply) throws IOException {
        _socketOut.println(cmd);
        String temp = _socketIn.readLine();
        if (temp == null) {
            throw new IOException ("IO error reading from socket, connection to server died?");
        }
        if (!temp.startsWith(String.valueOf(reply))) {
            throw new IOException ("Expected " + reply + ", got " + temp);
        }
    }
    
    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.server.util.FormatName class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

    /**
     * Check the server sends a 220 message, and
     * then send our HELO.
     *
     * @throws IOException if something goes wrong
     */
    private void init() throws IOException {
        String temp = _socketIn.readLine();
        // skip over any 220- lines
        while(temp != null && temp.startsWith("220-")) {
            temp = _socketIn.readLine();
        }
        // check if we got a 220 welcome header
        if(temp == null || !temp.startsWith("220 ")) {
            throw new IOException ("Server did not initialise with a 220 message");
        }
        // server gave a 220, it's ok to do a HELO
        sendCommand("HELO " + InetAddress.getLocalHost().getHostName(), 250);
    }

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = null;
    
    /**
     * A reference to the Reader connected to the server
     */
    private BufferedReader _socketIn;
    
    /**
     * A reference to the Writer connected to the server
     */
    private PrintWriter _socketOut;
    
    /**
     * A reference to the Socket connected to the server
     */
    private Socket _socket;
    
//---STATIC ATTRIBUTES---

}

